;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                 LCD-Routinen                ;;
;;                 ============                ;;
;;              (c)andreas-s@web.de            ;;
;;                                             ;;
;; 4bit-Interface                              ;;
;; DB4-DB7:       PD4-PD7                      ;;
;; RS:            PD0                          ;;
;; E:             PD1                          ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

 
 
 ;sendet ein Datenbyte an das LCD
lcd_data:
           push temp2
		   push temp3
		   
		   mov temp2, temp1             
           andi temp2, 0b11110000
		   sbi PORTD, 0       			;unteres Nibble auf Null setzen
           in temp3, PORTD
		   andi temp3,0b00001111
		   or temp3,temp2
		   out PORTD, temp3 
		               
           rcall lcd_enable             ;Enable-Routine aufrufen           
		   ;rcall delay50us                             ;2. Nibble, kein swap da es schon
           
		   swap temp1                            ;an der richtigen stelle ist
           andi temp1, 0b11110000       ;obere Hlfte auf Null setzen 
           sbi PORTD, 0	              				;entspricht 0b00010000
           in temp3, PORTD
		   andi temp3,0b00001111
		   or temp3,temp1
		   out PORTD, temp3
		                ;ausgeben
           rcall lcd_enable             ;Enable-Routine aufrufen
          
		   inc LCD_CursorPos
		
		   pop temp3
		   pop temp2              ;Delay-Routine aufrufen
           ret                          ;zurck zum Hauptprogramm

 ;sendet einen Befehl an das LCD
lcd_command:                            ;wie lcd_data, nur ohne RS zu setzen
           push temp2
		   push temp3
		   
		   mov temp2, temp1           
           andi temp2, 0b11110000
		   
		   cbi PORTD,0
		   in temp3,PORTD
		   andi temp3,0b00001111
		   or temp3,temp2
           out PORTD, temp3
		
           rcall lcd_enable
		   ;rcall delay50us
		   swap temp1
           andi temp1, 0b11110000

		   cbi PORTD,0
		   in temp3,PORTD
		   andi temp3,0b00001111
		   or temp3,temp1
           out PORTD, temp3
           rcall lcd_enable
           
			
			pop temp3
			pop temp2

           ret

 ;erzeugt den Enable-Puls
lcd_enable:
           sbi PORTD, 1                 ;Enable high
           rcall delay50us              ;3 Taktzyklen warten           
           cbi PORTD, 1                 ;Enable wieder low
           rcall delay50us
		   ret                          ;Und wieder zurck                     

 ;Pause nach jeder bertragung

delay50us: 
		   push temp1                             ;32us Pause
           ldi  temp1, $FF
delay50us_:dec  temp1
           brne delay50us_
		   pop temp1
           ret                          ;wieder zurck
  

 ;Lngere Pause fr manche Befehle
delay5ms:                               ;5ms Pause
           push temp2
		   push temp1
		   ldi  temp1, $C8
WGLOOP0:   ldi  temp2, $C8
WGLOOP1:   dec  temp2
           brne WGLOOP1
           dec  temp1
           brne WGLOOP0
		   pop temp1
		   pop temp2
           ret                          

 ;Initialisierung: muss ganz am Anfang des Programms aufgerufen werden
lcd_init:
           ldi	temp4,50
powerupwait:
           rcall	delay5ms			;Startup Time warten
           dec	temp4
           brne	powerupwait

           ldi temp1, 0b00110000        ;muss 3mal hintereinander gesendet           
		   out PORTD, temp1             ;werden zur Initialisierung
           rcall lcd_enable             ;1
           rcall delay5ms           
		   rcall lcd_enable             ;2
           rcall delay5ms           
		   rcall lcd_enable             ;und 3!
           rcall delay5ms
           ldi temp1, 0b00100000        ;4bit-Modus einstellen
           out PORTD, temp1
           rcall lcd_enable
           rcall delay5ms
           ldi temp1, 0b00101000        ;noch was einstellen...
           rcall lcd_command
           ldi temp1, 0b00001110        ;...nochwas...
           rcall lcd_command
           ldi temp1, 0b00000100        ;endlich fertig
           rcall lcd_command
           ret

 ;Sendet den Befehl zur Lschung des Displays
lcd_clear:
           push temp1
		   ldi temp1, 0b00000001   ;Display lschen
           rcall lcd_command
           rcall delay5ms
		   ldi LCD_CursorPos,0x80
		   pop temp1
           ret

; Setzt den Cursor auf die neue Position von temp1
CursorPos:
          mov temp1,LCD_CursorPos		      
           rcall lcd_command           
           ret



;Setzt den RAM-Adresszhler auf Pos:00
Zeile1:
           push temp1
		   ldi temp1, 0b10000000   ;Display auf RAM-Adr. 00 stellen
           mov LCD_CursorPos,temp1
		   rcall lcd_command           
		   
		   pop temp1
           ret


;Setzt den RAM-Adresszhler auf Pos:40
Zeile2:
           push temp1
		   ldi temp1, 0b11000000   ;Display auf RAM-Adr. 40 stellen
           mov LCD_CursorPos,temp1
		   rcall lcd_command           
			
		   pop temp1
           ret




;******************************************************************************		
AnzeigeZeile1:		
		
		ldi LCD_CursorPos,0x83
		rcall CursorPos		
		lds rBin1L,SSSollwert
		ldi rBin1H,0
		rcall Anz3
		
		ldi LCD_CursorPos,0x88
		rcall CursorPos	
		lds rBin1L,SSKp
		ldi rBin1H,0
		rcall Anz3

		ldi LCD_CursorPos,0x8C
		rcall CursorPos	
		lds rBin1L,SSKi
		ldi rBin1H,0
		rcall Anz3

		ldi LCD_CursorPos,0x90
		rcall CursorPos	
		lds rBin1L,SSKd
		ldi rBin1H,0
		rcall Anz3

		ret

AnzeigeZeile2:
		ldi LCD_CursorPos,0xC3
		rcall CursorPos	
		lds rBin1L,SSIstwert
		ldi rBin1H,0
		rcall Anz3

		ldi LCD_CursorPos,0xCA
		rcall CursorPos	
		lds rBin1L,SSErrAnzL
		lds rBin1H,SSErrAnzH
		rcall Anz3Sign

		ldi LCD_CursorPos,0xCF
		rcall CursorPos			
		lds rBin1L,SSOutL				;Pout laden und in ASCII Zeichen umwandeln
		lds rBin1H,SSOutH
		rcall Anz3Sign

		ret
		
;*******************************************************************************
	


Anz3Sign:
		SBRS rBin1H,7
		rjmp PosZahl3
NegZahl3:
		neg rBin1L
		com rBin1H
		ldi temp1,'-'
		rcall LCD_Data
		rjmp Anz3
PosZahl3:
		ldi rBin1H,0x00
		ldi temp1,' '
		rcall LCD_Data
		rjmp Anz3


Anz5Sign:
		SBRS rBin1H,7
		rjmp PosZahl5
NegZahl5:
		neg rBin1L
		com rBin1H
		ldi temp1,'-'
		rcall LCD_Data
		rjmp Anz5
PosZahl5:		
		ldi temp1,' '
		rcall LCD_Data
		rjmp Anz5



Anz3:	
		ldi zl,0x80
		ldi zh,0x00
		rcall Bin2ToAsc5
		ld temp1, z+
		ld temp1, z+					
		ld temp1, z+	
		rcall LCD_Data
		ld temp1, z+	
		rcall LCD_Data
		ld temp1, z+	
		rcall LCD_Data
		ret
		
Anz5:	ldi zl,0x80
		ldi zh,0x00
		rcall Bin2ToAsc5
		ld temp1, z+
		rcall LCD_Data			
		ld temp1, z+
		rcall LCD_Data			
		ld temp1, z+	
		rcall LCD_Data
		ld temp1, z+	
		rcall LCD_Data
		ld temp1, z+	
		rcall LCD_Data
		ret	
;*******************************************************************************


AusgTextZ1:
		push temp4
		push temp1		
		ldi	temp4,	20				; lade 20 in Register temp4
		ldi	zl,low(StringZ1*2)		; Zeige auf Z-Pointer Lowbyte von Tabelle1
		ldi	zh,high(StringZ1*2)		; Zeige auf Z-Pointer Highbyte von Tabelle1
loop_msg1:
		lpm	temp1,Z+				; lade temp1 mit Wert auf der der Z-Pointer zeigt 								
		rcall	lcd_data			; und erhhe Z-Pointer Daten senden
		dec	temp4					; Decrementiere temp4
		brne	loop_msg1			; vergleiche auf Null, wenn nicht springe		
		pop temp1
		pop temp4
		ret
	
AusgTextZ2:
		push temp4
		push temp1		
		ldi	temp4,	20				; lade 20 in Register temp4
		ldi	zl,low(StringZ2*2)		; Zeige auf Z-Pointer Lowbyte von Tabelle1
		ldi	zh,high(StringZ2*2)		; Zeige auf Z-Pointer Highbyte von Tabelle1
loop_msg2:
		lpm	temp1,Z+				; lade temp1 mit Wert auf der der Z-Pointer zeigt 								
		rcall	lcd_data			; und erhhe Z-Pointer Daten senden
		dec	temp4					; Decrementiere temp4
		brne	loop_msg2			; vergleiche auf Null, wenn nicht springe		
		pop temp1
		pop temp4
		ret

;********************************************************************************